part of tecfy_core_client_package;

class DashboardController extends BaseController {
  /// dashboard list
  static List<Dashboard>? dashboards;

  /// active dahbsrd
  Dashboard? activeDashboard;
  String filterDate = 'Year';
  RefreshController refreshController =
      RefreshController(initialRefresh: false); // for pull to refrsh

  DashboardController(State<StatefulWidget> state) : super(state) {
    getType();
  }

  @override
  dispose() {
    refreshController.dispose();
    super.dispose();
  }

  Future onItemTab(DashboardItemBase type) async {
    return await onItemTabEx(type, context);
  }

  static Future onItemTabEx(
      DashboardItemBase type, BuildContext context) async {
    print(type.onClickType);
    print(type.onClickEntity);
    print(type.onClickFilter?.map((e) => e.toJson()));
    if (type.onClickType == null || type.onClickType!.isEmpty) {
      return;
    }
    if (type.onClickType == 'EntityList') {
      AppNavigator.navigateTo(
          context,
          () =>
              EntityListPage(type.onClickEntity!, filters: type.onClickFilter));
      return;
    }
    if (type.onClickType == 'Report') {
      var result =
          await TecfyBasicApp.apiService.httpGetDynamic('Report/GetList');

      var report = result.data.firstWhere(
          (s) => s['dbName'] == type.onClickEntity,
          orElse: () => null);
      if (report == null) return;
      ReportDataService.onOpenReport(context, report);
      return;
    }
  }

  filterDateChanged(String val) {
    filterDate = val;
    refreshData();
    //setState();
  }

  getType() async {
    try {
      if (DashboardController.dashboards == null ||
          DashboardController.dashboards?.length == 0) {
        var typesResult = ConnectionService.activeConnection == null
            ? OperationResult(success: false)
            : await TecfyBasicApp.apiService
                .httpGetDynamic('Dashboard/GetAllTypes');
        if (!typesResult.success) {
          typesResult.data =
              jsonDecode(ConfigService.getValueString('DashboardAllType'));
          typesResult.success = typesResult.data != null;
        }
        if (typesResult.success) {
          ConfigService.setValueString(
              'DashboardAllType',
              jsonEncode(typesResult.data,
                  toEncodable: EntityService.customEncode));
          DashboardController.dashboards = typesResult.data
              .map<Dashboard>((d) => Dashboard.fromMap(d))
              .toList();
        }
      }

      activeDashboard = DashboardController.dashboards?.length == 0
          ? null
          : DashboardController.dashboards?[0];

      setState();
      refreshData(true);
    } catch (err, t) {
      print('XXXXXXXXXXXXX getType in dashboard  $err $t');
    }
  }

  Future<void> refreshData([bool fromCash = false]) async {
    if (activeDashboard == null) return;
    activeDashboard?.items.forEach((t) {
      if (t.isFrequentlyUpdated || !fromCash) t.value = null;
    });
    activeDashboard?.items.forEach((t) => getData(t, fromCash));
    await Future.delayed(Duration(seconds: 1));
    refreshController.refreshCompleted();
  }

  Future<dynamic> getData(DashboardItemBase type,
      [bool fromCash = false]) async {
    try {
      if (type.value != null) return type.value;
      if (type.value == -1) return null;
      type.value = -1;

      OperationResult result;
      if (fromCash &&
          ConfigService.getValueString(
                      'DashboardType/${type.dbName}$filterDate')
                  .length >
              0)
        result = OperationResult(success: false);
      else
        result = await TecfyBasicApp.apiService.httpGetDynamic(
            'Dashboard/GetData/${type.dbName}?filterDate=$filterDate&language=1');
      if (!result.success) {
        print(result);
        var chasheData = ConfigService.getValueString(
            'DashboardType${type.dbName}$filterDate');
        if (chasheData.isNotEmpty) {
          result.data = jsonDecode(chasheData);
          result.success = result.data != null;
        }
      }
      if (result.success) {
        ConfigService.setValueString('DashboardType${type.dbName}$filterDate',
            jsonEncode(result.data, toEncodable: EntityService.customEncode));
        type.value = result.data;
      } else {
        type.value = '0';
      }
      setState();
      return type.value;
    } catch (err, t) {
      print('Error getting dashboard $err $t');
      return null;
    }
  }
}
