part of tecfy_core_client_package;

class ManyToManyProperty extends StatefulWidget {
  /// list of selected value
  final List<dynamic>? selectedValues;

  /// entity object
  final Property property;

  /// label string
  final String? label;

  /// is requried
  final bool isRequired;

  /// on changed
  final void Function(List<dynamic>)? onChanged;
  ManyToManyProperty(
      {this.selectedValues,
      this.onChanged,
      required this.property,
      this.isRequired = false,
      this.label});
  @override
  _ManyToManyPropertyState createState() => _ManyToManyPropertyState();
}

class _ManyToManyPropertyState extends State<ManyToManyProperty> {
  List<dynamic>? _data;
  List<dynamic>? _initValue;
  List<dynamic> _selectedValues = [];
  String _displayFieldName = 'display';
  @override
  void initState() {
    if (widget.property is PropertyManyToMany) {
      var prop = widget.property as PropertyManyToMany;
      Entity? entity = EntityService.getEntity(prop.entityName);
      if (entity != null) {
        var displayFileds = entity.properties.where((e) => e.isDisplayField);
        if (displayFileds.length > 1) {
          var d = displayFileds
              .where((e) => e.language == LocalConfigService.language);
          if (d.length > 0) displayFileds = d;
        }
        if (displayFileds.length > 0)
          _displayFieldName = displayFileds.first.dbName;

        EntityService.getData(entity).then((result) {
          if (result.success) {
            _initValue = null;
            _data = result.data;
            _onSelect();
            setState(() {});
          } else
            result.showMessage(context);
        });
      } else {
        _displayFieldName = 'nameAr';
        _data = prop.options;
      }
    } else {
      print('XXXXXXXXXXXXXXXXX property is not many to many, not handled yet');
    }
    super.initState();
  }

  _onSelect() {
    if (_initValue == widget.selectedValues) return;
    _initValue = widget.selectedValues;
    if (_data == null ||
        _data!.length == 0 ||
        widget.selectedValues == null ||
        widget.selectedValues!.length == 0)
      _selectedValues = [];
    else
      _selectedValues = _data!
          .where((i) => widget.selectedValues!.contains(i['id']))
          .toList();
  }

  @override
  Widget build(BuildContext context) {
    _onSelect();
    return TextButton(
      //padding: EdgeInsets.all(0),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: <Widget>[
          SizedBox(height: _selectedValues.length == 0 ? 15.0 : 5.0),
          if (_selectedValues.length > 0)
            Padding(
                padding: EdgeInsets.only(bottom: 10.0),
                child: TecfyText(
                    '${widget.label}${widget.isRequired ? '*' : ''
                        ''}',
                    style: TextStyle(
                        color: Colors.black45,
                        // fontWeight: FontWeight.normal,
                        fontSize: 13))),
          _selectedValues.length == 0
              ? Padding(
                  padding: EdgeInsets.only(bottom: 10.0),
                  child: TecfyText(
                      '${widget.label}${widget.isRequired ? '*' : ''
                          ''}',
                      style: TextStyle(
                          fontSize: 20, fontWeight: FontWeight.normal)))
              : Wrap(
                  spacing: 3.0,
                  children: _selectedValues
                      .map((d) => Chip(label: TecfyText(d[_displayFieldName])))
                      .toList()),
          //style:TextStyle(fontSize: 18.0, color: Colors.black)),
          SizedBox(height: 5.0),
          Container(
            // margin: EdgeInsetsDirectional.only(start: 1.0, end: 1.0),
            height: 1.0,
            color: Colors.black,
          ),
          if (_selectedValues.length == 0) SizedBox(height: 20.0),
        ],
      ),
      onPressed: () {
        _openMultiSelectDialog();
      },
    );
  }

  _openMultiSelectDialog() {
    AppNavigator.showAppDialog(
        context,
        (_) => MultiSelectDialog(
              _data ?? [],
              _selectedValues,
              (d) {
                if (d != null) {
                  var dd = d.map((i) => i['id']).toList();
                  widget.onChanged?.call(dd);
                }
                setState(() {});
              },
              labelPropertyName: _displayFieldName,
              seachFunction: _displayFieldName == 'nameAr'
                  ? ((element, search) => search == null
                      ? true
                      : element[_displayFieldName].contains(search))
                  : null,
            ),
        animationType: AnimationTypes.slideUp);
  }
}
