part of tecfy_core_client_package;

class ReportListPage extends StatefulWidget {
  @override
  _ReportListPageState createState() => _ReportListPageState();
}

class _ReportListPageState extends State<ReportListPage> {
  List<dynamic>? _reports;
  List<dynamic>? _categories;

  @override
  void initState() {
    _getData();
    super.initState();
  }

  _getData() async {
    var resultCategories =
        await TecfyBasicApp.apiService.httpGetDynamic('Report/GetCategories');
    if (resultCategories.success) {
      setState(() {
        _categories = resultCategories.data;
      });
    }
    var result =
        await TecfyBasicApp.apiService.httpGetDynamic('Report/GetList');
    if (result.success) {
      if (TecfyBasicApp.isNewer(result.lastVersion)) {
        AppNavigator.navigateTo(context, () => UpgradePage(isRequired: false));
      }
      setState(() {
        _reports = result.data;
      });
    }
  }

  _onPressed(report) async {
    await ReportDataService.onOpenReport(context, report);
  }

  @override
  Widget build(BuildContext context) {
    return BasePage(
      context,
      'Reports',
      'report',
      isMobile: LayoutService.isMobile(context),
      appBarActions: [
        IconButton(
            onPressed: () {
              AppNavigator.navigateTo(
                  context, () => EntityListPage('TecfyReport'));
            },
            icon: Icon(Icons.settings))
      ],
      child: true
          ? _getGroupList()
          : _reports == null
              ? Center(child: AppNavigator.getLoadingWidget(100))
              : ListView(
                  children: _reports
                          ?.map<Widget>(
                            (report) => Container(
                              alignment: Alignment.center,
                              child: Container(
                                height: 50,
                                constraints: BoxConstraints(maxWidth: 300),
                                padding: EdgeInsets.all(5),
                                child: MaterialButton(
                                  shape: RoundedRectangleBorder(
                                      borderRadius: BorderRadius.circular(5)),
                                  padding: EdgeInsets.all(5),
                                  color: ThemeService.mainColor,
                                  onPressed: () => _onPressed(report),
                                  child: Row(
                                    mainAxisAlignment: MainAxisAlignment.start,
                                    children: <Widget>[
                                      SizedBox(width: 5),
                                      // isBussy
                                      //     ? Transform.scale(
                                      //         scale: 0.5,
                                      //         child: CircularProgressIndicator(
                                      //           valueColor: AlwaysStoppedAnimation<Color>(
                                      //               ThemeService.textOnMainColor),
                                      //         ))
                                      //     :
                                      Transform.scale(
                                          scale: 0.8,
                                          child: Icon(Icons.print,
                                              color: Colors.white)),
                                      SizedBox(width: 5),
                                      Text(
                                        report['name${LocalConfigService.languagePrefix}'] ??
                                            'N/A',
                                        style: TextStyle(
                                            color:
                                                ThemeService.textOnMainColor),
                                      )
                                    ],
                                  ),
                                ),
                              ),
                            ),
                          )
                          .toList() ??
                      [],
                ),
    );
  }

  String searchText = '';
  Widget _getGroupList() {
    if (_categories == null)
      return Center(child: AppNavigator.getLoadingWidget(100));
    return ConstrainedBox(
        constraints: BoxConstraints(maxWidth: 400),
        child: Column(children: <Widget>[
          SearchBox(
              placeholder: 'Search',
              onChange: (v) {
                searchText = v;
                setState(() {});
              }),
          SizedBox(height: 10.0),
          Expanded(
              child: ListView(
            padding: EdgeInsets.only(left: 20, right: 10),
            children: searchText.isNotEmpty
                ? _getSearchResult()
                : _categories!.map((g) => _getReports(g)).toList()
              ..add(_getReports(null)),
          )),
        ]));
  }

  List<Widget> _getSearchResult() {
    var reports = _reports!
        .where(
          (r) =>
              r['nameAr'].toString().contains(searchText) ||
              r['nameEn'].toString().contains(searchText) ||
              (r['descriptionAr'] ?? '').toString().contains(searchText) ||
              (r['descriptionEn'] ?? '').toString().contains(searchText),
        )
        .toList();
    if (reports.length == 0) return [Text('No Reports Found!')];
    return reports.map((e) => _getReportRow(e)).toList();
  }

  Widget _getReports(Map<String, dynamic>? group) {
    if (_reports == null)
      return Center(child: AppNavigator.getLoadingWidget(100));
    var reports =
        _reports!.where((r) => r["tecfyReportCategoryId"] == group?["id"]);
    if (reports.length == 0) return SizedBox();
    if (group == null)
      return Column(
        children: reports.map((report) => _getReportRow(report)).toList(),
      );
    return ExpansionTile(
      collapsedBackgroundColor: ThemeService.mainColor,
      collapsedIconColor: Colors.white,
      collapsedTextColor: Colors.white,
      title: Text(
        group['name${LocalConfigService.languagePrefix}'] ?? 'N/A',
        style: TextStyle(
            fontSize: 18.0,
            fontWeight: FontWeight.bold,
            decoration: TextDecoration.underline),
      ),
      children: reports.map((report) => _getReportRow(report)).toList(),
    );
  }

  Widget _getReportRow(report) {
    return ListTile(
      subtitle:
          report['description${LocalConfigService.languagePrefix}'] != null
              ? Text(
                  report['description${LocalConfigService.languagePrefix}'],
                  style: TextStyle(fontSize: 10),
                )
              : null,
      title: Text(report['name${LocalConfigService.languagePrefix}'] ?? 'N/A',
          style: TextStyle(fontSize: 18.0, fontWeight: FontWeight.bold)),
      onTap: () => _onPressed(report),
    );
  }
}
